#include <avr/io.h>
#include <util/delay.h>
#include <avr/sfr_defs.h>
#include <avr/pgmspace.h>

#include "lcd.h"
#include "usart.h"

#define s_down 	bit_is_clear(PINB, 1) // Switch Up to B1
#define s_up 	bit_is_clear(PINB, 2)	
#define s_enter bit_is_clear(PINB, 3)
#define s_clear bit_is_clear(PINB, 4)
#define s_bill	bit_is_clear(PINB, 5)

static const PROGMEM unsigned char welcome[] =
	{
	0x80,0x8E,0x91,0x91,0x91,0x91,0x91,0x89, // Sa - 1
	0x80,0x8C,0x92,0x82,0x82,0x82,0x82,0x81, // Sa - 2
	0x80,0x89,0x85,0x85,0x85,0x85,0x89,0x93, // Sa - 3 with sylable (swa)
	0x80,0x86,0x89,0x81,0x81,0x81,0x89,0x86, // Aa 
	0x80,0x84,0x88,0x90,0x90,0x91,0x89,0x86, // Ga -1
	0x80,0x86,0x89,0x91,0x91,0x81,0x81,0x82, // Ga -2
	0x80,0x86,0x89,0x92,0x92,0x92,0x92,0x89, // Tha -1
	0x80,0x8C,0x92,0x89,0x89,0x89,0x89,0x92 //  Tha - 2
	};
	// 0x6F at last. 0x6F denotes (Am) or o in malayalam.
//lcd_data(pgm_read_byte_near(&copyRightChar[i])); To read data values stored in program memory
static const PROGMEM unsigned char customsymbol[] =
	{
	0x80,0x80,0x80,0x80,0x80,0x91,0x8A,0x84, // 0-Down symbol custom .
	0x80,0x84,0x8A,0x84,0x8F,0x8C,0x84,0x8A, // 1-The shooting Guy
	0x82,0x86,0x8A,0x86,0x83,0x8E,0x82,0x85, // 2-Enemy no : 1
	0x86,0x89,0x85,0x82,0x8F,0x82,0x85,0x85, // 3-Enemy no : 3
	0x84,0x91,0x8A,0x94,0x85,0x8A,0x91,0x84, // 4-Explode Enemy
	0x84,0x8E,0x9F,0x80,0x80,0x9F,0x8E,0x84, // 5-UP / DOWN combination
	0x8E,0x91,0x80,0x9B,0x80,0x84,0x91,0x8E, // 6-Smiley
	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80
	};

void sendorder(char _data)	//Send one byte of Data
 {
 	/* 
		Now send a Packet
		Packet Format is AA<data><data inverse>Z
		total Packet size if 5 bytes.
		*/

		//Stabilize the Tx Module By Sending JUNK data
		UWriteData('J');	//J for junk

		//Send 'A'
		UWriteData('A');

		//Send Another 'A'
		UWriteData('A');

		//Send the data;
		UWriteData(_data);

		//Send inverse of data for error detection purpose
		UWriteData(~_data);

		//End the packet by writing 'Z'
		UWriteData('Z');

		//Wait for some time
		_delay_ms(4);
 }
 	

int main()
	{
	//Initialize the USART with Baud rate = 2400bps
	USARTInit(207); // edited by monster brain
	int update_bill = 0;
	int idle = 1;
	int i=0;  		// dummy variable
	uint8_t j,total_bill = 0;
	uint8_t k=0;
	int done=0;		// check the ORDER complete for every item 
	int upd_qty=1;  // quantity update in enter mode
	uint8_t temp;		// Temp variable
	int update = 1;	// Update LCD
	char code[10] = { 0x10,0x20,0x30,0x40,0x50,0x60,0x70,0x80,0x90 }; // Code to send to Reciever
	//char hotel[35]  = "Wireless Hotel Ordering System    "; // Hotel system name
  	char item[10][16]={"Tea","Coffee","porotta","Chappathi","Ch. Chilli", \
	"Veg Biriyani","Ch. Biriyani","Chicken Fry","Aloo Gobi","--End of Menu---"}; // Food items
	int rate[10] = { 5,6,5,4,55,35,50,30,40 };
	uint8_t order_item[10] = { 0,0,0,0,0,0,0,0,0,0 };
	uint8_t order_total = 0;
	uint8_t item_num[10] = {0,0,0,0,0,0,0,0,0,0};
	uint8_t total[10];
	////////////////////////
	uint8_t gameover=0;
	uint8_t score =0;
	uint8_t explode=2, shot=0;
	int pos_enemy1=17,pos_enemy2=20;
	uint8_t g_delay = 200;
	uint8_t by=0;
	int bx=-1;
	uint8_t bullet_move = 0;
	int p_pos = 0;
	////////////////////////
	InitLCD(0x00);		// Initialise LCD without Uline and Blink
	DDRB = DDRB & 0xC1; // Set the pins Port B1 - B5
	
	LCDCmd(0x0C);
	LCDCmd(0x40);
	_delay_ms(30);
	for(j=0;j<64;j+=1)
		{
			LCDData(pgm_read_byte_near(&welcome[j]));
			_delay_ms(10);
		}
	LCDClear();

	while(1)
	{	
		while(idle)
			{
			LCDGotoXY(4,0);
			for(j=0;j<8;j+=1)
				LCDData(0+j);
			LCDData(0x6F);
			_delay_ms(200);
			LCDGotoXY(2,1);
			LCDWriteString("WHO - System");
			_delay_ms(500);
			LCDClear();
			_delay_ms(50);
			if( (s_up)||(s_down)||(s_clear)||(s_enter)||(s_bill) )
				{	
					idle = 0;
					_delay_ms(50);
					LCDCmd(0x40);
					_delay_ms(30);
					for(j=0;j<64;j+=1)
					{	
						LCDData(pgm_read_byte_near(&customsymbol[j]));
						_delay_ms(10);
					}
					LCDClear();
				}
			}
		/*while(idle)	// Loop for displaying hotel System Name
			{
			LCDClear();
			_delay_ms(50);
			LCDWriteString(hotel);
			LCDGotoXY(0,1);
			LCDWriteString(" Press any key ");
			temp = hotel[0];
			for(i=0;i<34;i++)
				{
				hotel[i] = hotel[i+1];	// Swapping first and previous characters.
				}
			hotel[i-1] = temp;
			_delay_ms(200);
			if(s_up|s_down|s_enter|s_clear|s_order)
			 	{
				idle = 0;
				_delay_ms(500);
			 	}
			} */

		if(update) // Update LCD only if button is pressed
			{
			done = 0;
			LCDClear();
			_delay_ms(50);
			LCDWriteStringXY(0,0,">");
			LCDWriteString(item[i]);
			LCDWriteString("<");
			LCDWriteStringXY(15,0,"^");
			LCDWriteStringXY(0,1,item[i+1]);
			LCDGotoXY(15,1);
			if(i!=8)
				LCDData(0);
			update = 0;
			} // LCD update completed

		if(s_down)
		 {
		  	if(i<8)
			 {
			 	i += 1;
				_delay_ms(200);
				update = 1;
			 }			
		 }

		if(s_up)
		 {
		  	if(i>0) // check if it is top of menu
			 {
			 	i = i -1;
				_delay_ms(200);
			 	update = 1;
			 }
		 }

		if(s_enter)
		 { 	
		 	temp = item_num[i];	
			while(!done) // check order quantity is done
			{ 	
			if(upd_qty)  // Update LCD if quantity changed
				{
				_delay_ms(400);
				LCDClear();
				LCDGotoXY(0,0);
		 		LCDWriteString("Quantity? -");
	 			LCDGotoXY(13,0);
				LCDWriteInt(item_num[i],2);
				LCDData(5);
				LCDGotoXY(0,1);
				LCDWriteString("Price/pc - ");
				LCDWriteInt(rate[i],2);
				upd_qty=0;
				} // End LCD update
					if(s_up) // increase quantity by 1
						{
						item_num[i]+=1;
						_delay_ms(200);
						upd_qty = 1;
						}
					if(s_down) // decrease quantity by 1
						{
						if(item_num[i]>0)
							item_num[i]-=1;
						_delay_ms(200);
						upd_qty = 1;
						}
					if(s_clear) // clear the order
						{
						done = 1;
						item_num[i] = 0;
						LCDClear();
		   				LCDGotoXY(0,0);
		 				LCDWriteString("Cancelled");
						upd_qty = 1;
						_delay_ms(1000);
						upd_qty = 1;
						update = 1;
						}
					if(s_enter) // order Confirmed
						{	
						done=1;
						item_num[i]+= temp;
						LCDClear();
		   				LCDGotoXY(0,0);
		 				LCDWriteString("Done ");
						LCDData(6);
						total[i] = item_num[i] * rate[i];
						upd_qty = 1;
						code[i] = code[i] | item_num[i];
						LCDGotoXY(0,1);
						LCDWriteString("Sending...");
						_delay_ms(1000);
						update=	1;
						update_bill = 1;
						sendorder(code[i]);
						LCDWriteString("Send");	
						_delay_ms(500);
						} // completed the order
			} // while done loop. Quantity order Done.
		} // Enter Menu IF condition

		if(s_clear)
		{
			LCDClear();
			LCDWriteString("Game Loading ...");
			_delay_ms(2000);
			while(1) // Game starts
			{
				LCDClear();
				LCDGotoXY(0,p_pos);
				LCDData(1);
				if(bullet_move)
				{
					LCDGotoXY(bx,by);
					LCDData('-');
					bx += 1;
				}
				LCDGotoXY(pos_enemy1,0);
				LCDData(2);
				LCDGotoXY(pos_enemy2,1);
				LCDData(3);
				if(bx>15)
					bullet_move = 0;
				if(s_up)
					p_pos = 0;
				if(s_down)
					p_pos = 1;
				if(s_enter)
					{
					if(!bullet_move)
						{
						bullet_move = 1;
						bx = 1;
						}
					}
				pos_enemy1 -= 1;
				pos_enemy2 -= 1;
				if(!bullet_move)
					by = p_pos;
		
				if( (p_pos==0) && (pos_enemy1==0) )
					gameover = 1;
				if( (p_pos==1) && (pos_enemy2==0) )
					gameover = 1;
				if(explode==1)
					{
						LCDGotoXY(shot,1);
						LCDData(4);
						explode = 2;
					}
				if(explode==0)
					{
						LCDGotoXY(shot,0);
						LCDData(4);
						explode = 2;
					}			

				if( (!by) && ( (bx==pos_enemy1) || (bx+1)==pos_enemy1) )
					{ 
						bullet_move = 0;
						shot = bx;
						bx = -1;
						pos_enemy1 = 20;
						explode = 0;
						score += 1;
					}

				if( (by) && ( (bx==pos_enemy2) || (bx+1)==pos_enemy2) )
					{ 
						bullet_move = 0;
						shot = bx;
						bx = -1;
						explode = 1;
						pos_enemy2 = 18;
						score += 1;
					}			
	
				if(pos_enemy1 < 0)
					pos_enemy1 = 19;
				if(pos_enemy2 < 0)
					pos_enemy2 = 22;
		
		
				_delay_ms(g_delay);
				if(gameover)
					break;
			}
			LCDClear();
			LCDWriteString("Game Over");
			LCDWriteStringXY(0,1,"Score = ");
			LCDWriteInt(score,3);
			_delay_ms(2000);
			update = 1;
			/*LCDClear();
			_delay_ms(200);
			while(!(s_clear))
			{
				if(update_bill)
				{
					if(k==0)
						{
						LCDClear();
						_delay_ms(50);
						LCDGotoXY(0,0);
						LCDWriteString("-----Bill-----");
						for(j=0;j<order_total;j+=1)
							total_bill+=total[i];
						LCDGotoXY(0,1);
						LCDWriteString("TOTAL :");
						LCDGotoXY(13,1);
						LCDWriteInt(total_bill,3);
						}						
					else if(k<order_total)
						{
						LCDClear();
						_delay_ms(50);
						LCDGotoXY(0,0);
						LCDWriteString(item[order[k]]);
						LCDGotoXY(13,0);
						LCDWriteInt(total[order[k]],3);
						LCDWriteString(item[order[k+1]]);
						LCDGotoXY(13,1);
						LCDWriteInt(total[order[k+1]],3);
						}
						update_bill = 0;
				}
				if(s_up)
					{
						if(k>0)
							k-=1;
						_delay_ms(200);
						update_bill = 1;
					}
				if(s_down)
					{
						if(k<order_total-1)
							k+=1;
						_delay_ms(200);
						update_bill = 1;
					}
				}
			update = 1; */
		}
		if(s_bill)
		{
			update_bill = 1;
			k = 0;
			order_total = 0;
			j=0;
			for(k=0;k<9;k+=1)
			{
				if(item_num[k]>0)
				{
					order_item[j] = k;
					j += 1;
					order_total += 1;
					total_bill += total[k];
				}
			}
			k = 0;
			// Now we have ordered items in order_item array & total order items in order_total
			while(1)
			{
				if(update_bill)
				{	
					LCDClear();
					_delay_ms(100);
					//Display Bill details
					if(k==0)
					{
						LCDWriteString("------Bill------");
						LCDWriteStringXY(0,1,"Total - ");
						LCDWriteInt(total_bill,3);
						LCDWriteString(".00/-");
					}
					else
					{
						if(k<=order_total)
						{
							temp = order_item[k-1];
							LCDClear();
							LCDWriteString(item[temp]);
							LCDWriteString("-");
							LCDGotoXY(13,0);
							LCDWriteInt(total[temp],3);
							LCDGotoXY(0,1);
							LCDWriteString("Price:");
							LCDWriteInt(rate[temp],2);
							LCDWriteString(" Qty:");
							LCDWriteInt(item_num[temp],2);
						}
					}
					update_bill = 0;
				}
				if(s_up)
				{
					if(k>0)
					{	
					k-=1;
					update_bill = 1;
					}
				}
				if(s_down)
				{
					if(k<order_total)
					{
						k+=1;
						update_bill =1;
					}
				}
				if(s_clear)
					break;
				update = 1;
				_delay_ms(200);
			}
		}
	} // the Main Loop
}
